package org.mmga.clubs.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.mmga.clubs.annotations.AuthorizationRequired;
import org.mmga.clubs.entities.BaseResponse;
import org.mmga.clubs.entities.Pager;
import org.mmga.clubs.entities.user.*;
import org.mmga.clubs.entities.user.change.UserChangeAuthVo;
import org.mmga.clubs.entities.user.change.UserChangePasswordVo;
import org.mmga.clubs.entities.user.change.UserRenameVo;
import org.mmga.clubs.entities.verify.VerifyResponseVo;
import org.mmga.clubs.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

@RestController
@RequestMapping("/api/user")
@Tag(name = "用户", description = "用户相关接口")
@Slf4j
public class UserController {
    private final UserService service;
    @Autowired
    public UserController(UserService userService){
        this.service = userService;
    }
    @PostMapping("/login")
    @Operation(description = "用户登录", responses = {@ApiResponse(description = "返回是否登录成功", responseCode = "200")})
    public BaseResponse<Boolean> login(@RequestBody UserLoginVo user, HttpServletResponse response){
        return service.login(user, response);
    }
    @PutMapping("/create")
    @Operation(description = "创建用户", responses = {@ApiResponse(description = "返回创建后的用户")})
    public BaseResponse<Boolean> createUser(@RequestBody UserRegVo user, HttpServletResponse response){
        return service.createUser(user, response);
    }
    @GetMapping("/info")
    @Operation(description = "获取用户信息")
    public BaseResponse<User> getUserInfo(@RequestAttribute("user") int userId){
        return service.getUserInfo(userId);
    }
    @GetMapping("/all")
    @Operation(description = "获取所有用户信息（分页）")
    @AuthorizationRequired(requiredPermissionId = 4)
    public BaseResponse<Pager<User>> getAllUserInfo(@RequestParam("num") int num, @RequestParam("page") int page){
        return service.getAllUserInfo(num, page);
    }
    @PutMapping("/rename")
    @Operation(description = "修改用户名")
    @AuthorizationRequired
    public BaseResponse<Boolean> changeUsername(@RequestBody UserRenameVo renameVo, @RequestAttribute("user") int userId){
        return service.changeUsername(renameVo, userId);
    }
    @PutMapping("/password")
    @Operation(description = "修改密码")
    @AuthorizationRequired
    public BaseResponse<Boolean> changePassword(@RequestBody UserChangePasswordVo changePasswordVo, @RequestAttribute("user") int userId){
        return service.changePassword(changePasswordVo, userId);
    }
    @PutMapping("/auth")
    @Operation(description = "修改用户权限组")
    @AuthorizationRequired(requiredPermissionId = 4)
    public BaseResponse<Boolean> changeAuth(@RequestBody UserChangeAuthVo userChangeAuthVo){
        return service.changeAuth(userChangeAuthVo);
    }
    @PostMapping("/avatar")
    @Operation(description = "上传头像")
    @AuthorizationRequired
    public BaseResponse<String> changeAvatar(MultipartFile file, @RequestAttribute("user") int userId){
        return service.changeAvatar(file, userId);
    }
    @PutMapping("/avatar")
    @Operation(description = "修改用户头像")
    @AuthorizationRequired
    public BaseResponse<Boolean> changeAvatar(@RequestParam("code") @Schema(description = "修改头像操作码，可以通过将图片文件上传至") String avatarOperationCode, @RequestAttribute("user") int uid){
        return service.changeAvatar(avatarOperationCode, uid);
    }
    @GetMapping("/verify")
    @Operation(description = "获取验证码")
    public BaseResponse<VerifyResponseVo> generatorVerifyCode(){
        return service.getVerifyCode();
    }
    @GetMapping("/avatar/{sha1}")
    @Operation(description = "获取头像文件")
    public void getAvatar(HttpServletResponse response, @Schema(description = "头像文件SHA1值") @PathVariable("sha1") String sha1){
        service.getAvatar(response, sha1);
    }
    @GetMapping("/info/{id}")
    @Operation(description = "获取简略用户信息")
    public BaseResponse<User> getSimpleInfo(@PathVariable("id") int userId){
        return service.getUserInfo(userId);
    }
}

