package org.mmga.clubs.entities.chess;


import com.alibaba.fastjson2.annotation.JSONField;
import jakarta.websocket.Session;
import lombok.Data;
import lombok.RequiredArgsConstructor;
import org.mmga.clubs.entities.chess.packet.PlayerJoinRoomPacket;
import org.mmga.clubs.entities.chess.packet.PlayerLeavePacket;
import org.mmga.clubs.entities.chess.packet.PlayerSideAllocationPacket;
import org.mmga.clubs.entities.chess.packet.RoomInfoPacket;
import org.mmga.clubs.entities.user.User;
import org.mmga.clubs.utils.WebSocketUtils;

import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.UUID;

@Data
public class Room {
    private UUID id;
    private RoomState state;
    @JSONField(serialize = false, deserialize = false)
    private Session whiteSession;
    @JSONField(serialize = false, deserialize = false)
    private Session blackSession;
    @JSONField(serialize = false, deserialize = false)
    private byte[][] pieces;
    private boolean isWhiteAcceptRestart;
    private boolean isBlackAcceptRestart;
    private boolean canWhiteDown;

    public Room() {
        this.id = UUID.randomUUID();
        this.state = RoomState.CREATED;
        resetPieces();
    }

    public void resetPieces() {
        this.canWhiteDown = false;
        this.pieces = new byte[16][16];
        for (int i = 0; i < 16; i++) {
            for (int j = 0; j < 16; j++) {
                this.pieces[i][j] = -1;
            }
        }
        isWhiteAcceptRestart = false;
        isBlackAcceptRestart = false;
    }

    public boolean isFull() {
        return Objects.nonNull(this.whiteSession) && Objects.nonNull(this.blackSession);
    }

    public Optional<String> join(Session session, User user) {
        if (this.getState() == RoomState.GAMING) {
            return Optional.of("你所加入的房间正在游戏！");
        }
        if (this.isFull()) {
            return Optional.of("所加入的房间已满");
        }
        if (Objects.nonNull(this.whiteSession)) {
            WebSocketUtils.sendPacket(new PlayerSideAllocationPacket(false), session);
            WebSocketUtils.sendPacketIfPossible(new PlayerJoinRoomPacket(user, this), whiteSession);
            this.blackSession = session;
            this.setState(RoomState.GAMING);
        } else {
            WebSocketUtils.sendPacket(new PlayerSideAllocationPacket(true), session);
            WebSocketUtils.sendPacketIfPossible(new PlayerJoinRoomPacket(user, this), blackSession);
            this.whiteSession = session;
            this.setState(RoomState.WAITING);
        }
        return Optional.empty();
    }

    public void leave(Session session) {
        String sessionId = session.getId();
        if (Objects.nonNull(this.blackSession) && this.blackSession.getId().equals(sessionId)) {
            WebSocketUtils.sendPacket(new PlayerLeavePacket(sessionId), whiteSession);
            this.blackSession = null;
            this.setState(RoomState.WAITING);
        }
        if (Objects.nonNull(this.whiteSession) && this.whiteSession.getId().equals(sessionId)) {
            WebSocketUtils.sendPacket(new PlayerLeavePacket(sessionId), blackSession);
            this.whiteSession = null;
            this.setState(RoomState.WAITING);
        }
    }

    public RoomInfoPacket getRoomInfo(Map<String, User> users) {
        Session whiteSession = this.getWhiteSession();
        Session blackSession = this.getBlackSession();
        User whiteUser = whiteSession != null ? users.get(whiteSession.getId()) : null;
        User blackUser = blackSession != null ? users.get(blackSession.getId()) : null;
        return new RoomInfoPacket(this.id, whiteUser, blackUser, this.state, pieces);
    }

    public void broadcast(Object packet) {
        if (packet == null) return;
        WebSocketUtils.sendPacketIfPossible(packet, whiteSession);
        WebSocketUtils.sendPacketIfPossible(packet, blackSession);
    }

    public boolean isIn(String sessionId) {
        return (Objects.nonNull(whiteSession) && whiteSession.getId().equals(sessionId)) || (Objects.nonNull(blackSession) && blackSession.getId().equals(sessionId));
    }

    public Optional<String> downPiece(int x, int y, String sessionId) {
        if (!this.getState().equals(RoomState.GAMING)) return Optional.of("游戏暂未开始！");
        byte originalValue = this.pieces[y][x];
        if (originalValue != -1) return Optional.of("此位置已落子！");
        if (whiteSession.getId().equals(sessionId)) {
            if (!canWhiteDown) return Optional.of("请等待对手落子");
            canWhiteDown = false;
            this.pieces[y][x] = 1;
        } else {
            if (canWhiteDown) return Optional.of("请等待对手落子");
            canWhiteDown = true;
            this.pieces[y][x] = 0;
        }
        return Optional.empty();
    }

    public Optional<WinInfo> isWin() {
        for (int y = 0; y < 16; y++) {
            for (int x = 0; x < 16; x++) {
                byte startPointType = this.pieces[y][x];
                if (startPointType == -1) continue;
                for (Faces value : Faces.values()) {
                    boolean facePossible = true;
                    for (int i = 1; i < 5; i++) {
                        int totalXDelta = value.xDelta * i;
                        int totalYDelta = value.yDelta * i;
                        int fullyX = x + totalXDelta;
                        int fullyY = y + totalYDelta;
                        if (fullyX < 0 || fullyX > 15 || fullyY < 0 || fullyY > 15) {
                            facePossible = false;
                            break;
                        }
                        if (this.pieces[fullyY][fullyX] != startPointType) {
                            facePossible = false;
                            break;
                        }
                    }
                    if (!facePossible) continue;
                    return Optional.of(new WinInfo(value, x, y, startPointType == 1));
                }
            }
        }
        return Optional.empty();
    }

    public void requestRestart(Session session) {
        String sessionId = session.getId();
        if (sessionId.equals(whiteSession.getId())) {
            isWhiteAcceptRestart = true;
        }
        if (sessionId.equals(blackSession.getId())) {
            isBlackAcceptRestart = true;
        }
    }

    public void replace(String existsUser, Session session) {
        if (Objects.nonNull(this.whiteSession) && whiteSession.getId().equals(existsUser)) {
            whiteSession = session;
        }
        if (Objects.nonNull(this.blackSession) && blackSession.getId().equals(existsUser)) {
            blackSession = session;
        }
    }

    public boolean isPlayerWhite(String sessionId) {
        return sessionId.equals(whiteSession.getId());
    }

    public enum RoomState {
        CREATED,
        WAITING,
        GAMING
    }

    @RequiredArgsConstructor
    public enum Faces {
        UP(0, 1),
        UP_RIGHT(1, 1),
        RIGHT(1, 0),
        DOWN_RIGHT(1, -1),
        DOWN(0, -1),
        DOWN_LEFT(-1, -1),
        LEFT(-1, 0),
        UP_LEFT(-1, 1);
        public final int xDelta, yDelta;
    }

    public record WinInfo(Faces face, int originalX, int originalY, boolean isWhite) {
    }
}
